#include <IRremote.h>
#include <Adafruit_NeoPixel.h>

#define FORWARD     0xFF02FD   
#define BACKWARD    0xFF9867   
#define TURNLEFT    0xFFE01F   
#define TURNRIGHT   0xFF906F   
#define STOP        0xFFA857   
#define Buzzer      0xFF7A85 
#define Avo_Obs     0xFF18E7
#define TRACK       0xFF30CF  

//Define the direction of car movement
#define  stop     0
#define  forward  1
#define  back     2
#define  left     3
#define  right    4

//Three-way tracking module pin definition
#define LeftPin      A0
#define MiddlePin    A1
#define RightPin     A2

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN 13 // On Trinket or Gemma, suggest changing this to 1

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS 5 // Popular NeoPixel ring size

// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

static void (*reset_this_CPU)(void) = 0x0000;

#define DELAYVAL 500 // Time (in milliseconds) to pause between pixels

int MotSpeed1 = 200; 
int MotSpeed2 = 200;

IRrecv irrecv(2); 
decode_results results; 

void setup()
{
    Serial.begin(9600); 
    irrecv.enableIRIn();

    pinMode(5, OUTPUT);
    pinMode(6, OUTPUT);
    pinMode(9, OUTPUT);
    pinMode(10, OUTPUT);
    
    pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)

}

void loop()
{
   
   IR();
     
}

void IR()
{
    if (irrecv.decode(&results) ) //Check whether infrared remote control signal is received
    { 

        unsigned long res_val = results.value; // The received value is stored in res_val 
        
        Serial.print("res_val=");
        Serial.println(res_val,HEX);// Output instruction information

        irrecv.resume();// Wait for the next reception

        switch(res_val)
        {
           
            case FORWARD:           
            motor(forward,MotSpeed1,MotSpeed2);
            break;

            case BACKWARD:             
            motor(back,MotSpeed1,MotSpeed2);         
            break;  

           case TURNLEFT:           
            motor(left,MotSpeed1,MotSpeed2);
            break;

            case TURNRIGHT:             
            motor(right,MotSpeed1,MotSpeed2);         
            break;    

            case STOP:           
            motor(stop,0,0 );          
            break; 

            case TRACK:           
            patrol();        
            break;  
        }   
    }
}

/*
M3 M4 motor drive
M3 ------> speed1
M4 ------> speed2
Speed adjustment range: 0~255
*/
void motor(int dir ,int speed1, int speed2)
{  
    switch (dir)
    {
    case stop:
        motor3(0, 0);
        motor4(0, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case forward:
       
        motor3(speed1, 0);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 150, 0));
        pixels.setPixelColor(1, pixels.Color(0, 150, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case back:
        
        motor3(0, speed1);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(200, 0, 0));
        pixels.setPixelColor(1, pixels.Color(200, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case left:
        motor3(0, speed1);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0)); //LED out on the right
        pixels.setPixelColor(1, pixels.Color(255, 255, 0)); //The left LED lights up
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case right:
        
        motor3(speed1, 0);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(255, 255, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;
    
    default:
        break;
    }
}

void motor1(bool PinState1, bool PinState2)
{
   digitalWrite(12,PinState1);
   digitalWrite(4,PinState2); 
}

void motor2(bool PinState1, bool PinState2)
{
   digitalWrite(7,PinState1);
   digitalWrite(8,PinState2); 
}

void motor3(int speed1, int speed2)
{
   analogWrite(5,speed1);
   analogWrite(6,speed2); 
}

void motor4(int speed1, int speed2)
{
   analogWrite(9,speed1);
   analogWrite(10,speed2); 
}

void patrol()
{
    if(analogRead(LeftPin) > 100 && analogRead(MiddlePin) < 500 && 
       analogRead(RightPin) < 500)
    {
        motor(left,180,180);   
    }
    else if(analogRead(LeftPin) < 400 && analogRead(MiddlePin) > 100 &&
            analogRead(RightPin) < 500)
    {
        motor(forward,160,160);        
    }
    else if(analogRead(LeftPin) < 400 && analogRead(MiddlePin) < 400 && 
            analogRead(RightPin) > 100)
    {
        motor(right,180,180); 
    }
    else motor(stop,0,0);  
} 
