
//******************5.23***********************//
#include <IRremote.h>
#include <Adafruit_NeoPixel.h>

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN 13 // On Trinket or Gemma, suggest changing this to 1

#define NUMPIXELS 5 // Popular NeoPixel ring size
// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
 Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

static void (*reset_this_CPU)(void) = 0x0000;


//Define infrared remote coding function
#define FORWARD     0xFF02FD   
#define BACKWARD    0xFF9867   
#define TURNLEFT    0xFFE01F   
#define TURNRIGHT   0xFF906F   
#define STOP        0xFFA857   
#define Buzzer      0xFF7A85 
#define Avo_Obs     0xFF18E7
#define TRACK       0xFF30CF   

//Define the direction of car movement
#define  stop     0
#define  forward  1
#define  back     2
#define  left     3
#define  right    4

// Define ultrasonic sensor pins
#define  Trig A4 
#define  Echo A5

//Three-way tracking module pin definition
#define LeftPin      A0
#define MiddlePin    A1
#define RightPin     A2

#define servoPin     3
#define BEEP         11



unsigned long currentMillis;

unsigned long ServoTime = 0;


//Set motor default speed
int MotSpeed1 = 180; 
int MotSpeed2 = 180;

IRrecv irrecv(2); 
decode_results results; 
//Servo myservo;

void setup()
{
    Serial.begin(9600); 
    irrecv.enableIRIn();
    // The initial Angle of the steering gear
    servo(55); 
    //Set the pin mode
    for (int i = 4; i <= 12; i++)
    {
        pinMode(i, OUTPUT);
        digitalWrite(i,LOW);
    }
    pinMode(LeftPin,INPUT);
    pinMode(MiddlePin,INPUT);
    pinMode(RightPin,INPUT);
    pinMode(Trig,OUTPUT);   
    pinMode(Echo,INPUT);    

    pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)
    //delay(1000);
    //delay(1000);
    //delay(1000);
    beep();
    Init();
    Serial.println("Initialization completed!");
}

void loop()
{
    currentMillis = millis();

     
    IR();
    bluetooth();
    PrintData();
    //PrintData();
     
    // pixels.Color() takes RGB values, from 0,0,0 up to 255,255,255
    // Here we're using a moderately bright green color:
}

/*
M3 M4 motor drive
M3 ------> speed1
M4 ------> speed2
Speed adjustment range: 0~255
*/
void motor(int dir ,int speed1, int speed2)
{  
    switch (dir)
    {
    case stop:
        motor3(0, 0);
        motor4(0, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case forward:
       
        motor3(speed1, 0);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 150, 0));
        pixels.setPixelColor(1, pixels.Color(0, 150, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case back:
        
        motor3(0, speed1);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(200, 0, 0));
        pixels.setPixelColor(1, pixels.Color(200, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case left:
        motor3(0, speed1);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0)); //LED out on the right
        pixels.setPixelColor(1, pixels.Color(255, 255, 0)); //The left LED lights up
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case right:
        
        motor3(speed1, 0);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(255, 255, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;
    
    default:
        break;
    }
}

void motor1(bool PinState1, bool PinState2)
{
   digitalWrite(12,PinState1);
   digitalWrite(4,PinState2); 
}

void motor2(bool PinState1, bool PinState2)
{
   digitalWrite(7,PinState1);
   digitalWrite(8,PinState2); 
}

void motor3(int speed1, int speed2)
{
   analogWrite(5,speed1);
   analogWrite(6,speed2); 
}

void motor4(int speed1, int speed2)
{
   analogWrite(9,speed1);
   analogWrite(10,speed2); 
}

void bluetooth()
{
    char serialData;   
    static char cmd;

    if(Serial.available() > 0)      
    {
        serialData = Serial.read(); 
        Serial.println(serialData); 
       
        SetSeep(serialData);

        if ('F' == serialData )  
        {
            motor(forward,MotSpeed1,MotSpeed2);
        }     
        
        else if('B' == serialData )  
        {
            motor(back,MotSpeed1,MotSpeed2);
        }
        else if ('L' == serialData )  
        {
            motor(left,MotSpeed1,MotSpeed2);
        }     
        
        else if('R' == serialData )  
        {
            motor(right,MotSpeed1,MotSpeed2);
        }
        
        else if('S' == serialData )  
        {
            motor(stop,0,0);
            cmd = 'S';
        } 
        else if('T' == serialData )  
        {
            cmd = 'T';
        }
        else if('A' == serialData )  
        {
            cmd = 'A';
        }
          else if('C' == serialData )  
        {
            cmd = 'C';
        }
        else if('K' == serialData )  
        {
            reset_this_CPU();   //Restart the program
        }
         
    }
    
    switch(cmd)
    {
        case 'T':           
        patrol(); 
        break;

        case 'A':             
        avoidance(20);        
        break;  
        
        case 'C':           
         track();
        break; 
    }  
}     

void IR()
{
  //Check whether infrared remote control signal is received
    if (irrecv.decode(&results) ) 
    { 
        // The received value is stored in res_val 
        unsigned long res_val = results.value; 
        
        Serial.print("res_val=");
        // Output instruction information
        Serial.println(res_val,HEX);

        irrecv.resume();// Wait for the next reception

        switch(res_val)
        {
            case FORWARD:           
            motor(forward,MotSpeed1,MotSpeed2);
            break;

            case BACKWARD:             
            motor(back,MotSpeed1,MotSpeed2);         
            break;  

           case TURNLEFT:           
            motor(left,MotSpeed1,MotSpeed2);
            break;

            case TURNRIGHT:             
            motor(right,MotSpeed1,MotSpeed2);         
            break;    

            case STOP:           
            motor(stop,0,0 );          
            break; 

            case TRACK:           
            patrol();        
            break;  
        }   
    }
}

/*
Function: obtain ultrasonic sensor ranging data
Parameters: Trig, Echo
Parameter description: sensor connected to the motherboard pin port A4,A5
Trig -------> pin A4
Echo -------> pin A5
*/
float GetDistance()
{
    float distance;
  
	digitalWrite(Trig, LOW); 
	delayMicroseconds(2); 
	digitalWrite(Trig, HIGH); 
	delayMicroseconds(10);
	digitalWrite(Trig, LOW);

   distance = pulseIn(Echo, HIGH) / 58.00;
     
	return distance;
}

/*
* Function: Obstacle avoidance
* Parameter: set_dis sets the obstacle avoidance distance
*/
void avoidance(int set_dis)
{
    int angle;
    int dis[3];// distance
  
    motor(forward,MotSpeed1,MotSpeed2);

    servo(48); //Steering engine back to center
    //Obtain the distance between the cart and the obstacle and store it in dis[1]
    dis[1] = GetDistance(); 

    if(dis[1] < set_dis )
    {
        motor(stop,0,0 );   //Stop the car

        //If the left and right infrared obstacle avoidance sensors do not encounter obstacles when the steering gear rotates
        
            for (angle = 48; angle <= 100; angle++) 
            {
                servo(angle);
                delay(1);         
            }
           // Measure the distance between the left obstacle and the cart, and store the measurement data in dis[2]
            dis[2]=GetDistance(); 

            for (angle = 48; angle >= 10; angle--) 
            {
                servo(angle);  
                delay(1);             
                if(angle == 48)  
                {   // Stores measurement data in dis[1]
                    dis[1] = GetDistance(); 
                } 
            }
           // Record the range data on the right side of the trolley
            dis[0] = GetDistance(); 
            for (angle = 10; angle <= 48; angle++) 
            {
                servo(angle);
                delay(1); 
            }
        
        if(dis[0] < dis[2] ) 
        {
            motor(back,MotSpeed1,MotSpeed2); // back
            delay(350);
            motor(left,MotSpeed1,MotSpeed2);  //left
            delay(350);
        }
        //The right is more distant from the obstacle than the left
        else if (dis[0] > dis[2] )
        {
            motor(back,MotSpeed1,MotSpeed1);
            delay(350);
            motor(right,MotSpeed1,MotSpeed2);  
            delay(350);
        } 
    }
}

void track()
{
    int get_dis = 0;
    int pos;   // Steering gear Angle

    get_dis = GetDistance();

    
   // The distance is between 20cm and 40cm
    if(get_dis > 20 && get_dis < 40)
    {
         motor(forward,MotSpeed1,MotSpeed2);
    }
    // Backup less than 15cm
    else  if(get_dis < 15 )
    {
        motor(back,MotSpeed1,MotSpeed1); 
    } 
    else  
    {
       
        motor(stop,0,0 ); 
    }  
}

void RGBColor(int count,int R,int G,int B)
{
    pixels.clear(); // Set all pixel colors to 'off'
    for (int i = count; i < 2; i++)
    {
        pixels.setPixelColor(i, pixels.Color(R, G, B));
    }
    
    pixels.show();   // Send the updated pixel colors to the hardware.
}

void patrol()
{   //
    if(analogRead(LeftPin) > 100 && analogRead(MiddlePin) < 500 && 
       analogRead(RightPin) < 500)
    {
        motor(left,MotSpeed1,MotSpeed2);   
    }
    else if(analogRead(LeftPin) > 100 && analogRead(MiddlePin) > 100 && 
       analogRead(RightPin) < 500)
    {
        motor(left,MotSpeed1,MotSpeed2);   
    }
    else if(analogRead(LeftPin) < 400 && analogRead(MiddlePin) > 100 &&
            analogRead(RightPin) < 500)
    {
        motor(forward,MotSpeed1,MotSpeed2);        
    }
    else if(analogRead(LeftPin) > 100 && analogRead(MiddlePin) > 100 &&
            analogRead(RightPin) > 100)
    {
        motor(forward,MotSpeed1,MotSpeed2);        
    }
    else if(analogRead(LeftPin) < 400 && analogRead(MiddlePin) < 400 && 
            analogRead(RightPin) > 100)
    {
        motor(right,MotSpeed1,MotSpeed2); 
    }

    else if(analogRead(LeftPin) < 400 && analogRead(MiddlePin) > 100 && 
            analogRead(RightPin) > 100)
    {
        motor(right,MotSpeed1,MotSpeed2); 
    }
        else if(analogRead(LeftPin) < 100 && analogRead(MiddlePin) < 100 && 
            analogRead(RightPin) < 100)
    {
        motor(forward,MotSpeed1,MotSpeed2); 
    }
    
    else motor(stop,0,0);  
}

void PrintData()
{
    static unsigned long PrintTime = 0;

    //Print information every second
    if (currentMillis - PrintTime >= 1000)
    {
        PrintTime = currentMillis;     
        Serial.print("left = ");
        Serial.println(analogRead(LeftPin));
        Serial.print("middle = ");
        Serial.println(analogRead(MiddlePin));
        Serial.print("right = ");
        Serial.println(analogRead(RightPin));
        MeasurVoltage();
    } 
}

//Customize a steering gear function
void servo(int angle) 
{ 
    //Sends 5 pulses
    for(int i=0;i<5;i++)
    {
      //Convert the Angle to a pulse width of 500-2480
        int pulsewidth = (angle * 11) + 500; 
        //Turn steering gear interface level to high
        digitalWrite(servoPin, HIGH);
        //The number of microseconds of delay pulse width value   
        delayMicroseconds(pulsewidth);  
        //Turn steering gear interface level to low
        digitalWrite(servoPin, LOW);    
        delayMicroseconds(20000 - pulsewidth);
    }
}

/*

*/
void MeasurVoltage()
{
    float val = 0;
    float voltage ;
    float sum = 0;      //The measurement results are added up
    int SumNum = 100;  //Number of measurement
    static unsigned long VolTime = 0;

    val = analogRead(A3);

    voltage = val / 39.09;  //Get battery voltage
    
    for (int i = 0; i < SumNum; i++)
    {
        sum += voltage; 
    }
    //Calculate average value, get more accurate measurement voltage value
    voltage = sum / SumNum;  
    

    Serial.print("Battery voltage:");   
    Serial.println(voltage);  
 
    if( (voltage < 3.6) && (voltage > 3.30))
    {
        Serial.println("Battery voltage is too low, please charge!");
    }
    else if(voltage < 3.30)
    {
        
        Serial.println("Voltage is too low, may not work properly, please charge!");
    }
}

void SetSeep(char val)
{
    if(val == '+')
    {
        if( MotSpeed1 < 255)
        {
            MotSpeed1 += 10;
            MotSpeed2 = MotSpeed1;

            if(MotSpeed1 > 255) //Prevent exceeding 255
            {
                MotSpeed1 = 255;
                MotSpeed2 = 255;
            }  
        }
        Serial.print("speed = ");  
        Serial.println(MotSpeed1); 
    }
    if (val == '-')
    {
        if( MotSpeed1 > 100) //Limit speed minimums
        {
            MotSpeed1 -= 10;
            MotSpeed2 = MotSpeed1;
        }

        Serial.print("speed = ");  
        Serial.println(MotSpeed1);  
    }
}
        
void  Init()
{
    pixels.clear();
    
    pixels.setPixelColor(0, pixels.Color(0, 0, 0));
    pixels.setPixelColor(1, pixels.Color(0, 0, 0));
    pixels.setPixelColor(0, pixels.Color(0, 150, 0));
    pixels.setPixelColor(1, pixels.Color(0, 150, 0));
    delay(500);
    pixels.setPixelColor(0, pixels.Color(150, 0, 0));
    pixels.setPixelColor(1, pixels.Color(150, 0, 0));
    delay(500);
     pixels.setPixelColor(0, pixels.Color(0, 0, 0));
    pixels.setPixelColor(1, pixels.Color(0, 0, 0));
    pixels.show(); 
}

void beep()
{   
    for(int j=0;j<500;j++)
    {
        digitalWrite(BEEP,HIGH);
        delay(1);
        digitalWrite(BEEP,LOW);
        delay(1);
    }   
}
