#include <Adafruit_NeoPixel.h>

//Define the direction of car movement
#define  stop     0
#define  forward  1
#define  back     2
#define  left     3
#define  right    4

#define  Trig A4 
#define  Echo A5

#define servoPin     3

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN 13 // On Trinket or Gemma, suggest changing this to 1

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS 5 // Popular NeoPixel ring size

// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

static void (*reset_this_CPU)(void) = 0x0000;

#define DELAYVAL 500 // Time (in milliseconds) to pause between pixels

int MotSpeed1 = 200; 
int MotSpeed2 = 200;

void setup()
{
    Serial.begin(9600); 

    pinMode(5, OUTPUT);
    pinMode(6, OUTPUT);
    pinMode(9, OUTPUT);
    pinMode(10, OUTPUT);
    pinMode(Trig,OUTPUT);   
    pinMode(Echo,INPUT); 
    // Define ultrasonic sensor pins
    

    pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)

}

void loop()
{
   
   avoidance(30);
   //GetDistance();
     
}

/*
Function: obtain ultrasonic sensor ranging data
Parameters: Trig, Echo
Parameter description: sensor connected to the motherboard pin port A4,A5
Trig -------> pin A4
Echo -------> pin A5
*/
float GetDistance()
{
    float distance;
  
	digitalWrite(Trig, LOW); 
	delayMicroseconds(2); 
	digitalWrite(Trig, HIGH); 
	delayMicroseconds(10);
	digitalWrite(Trig, LOW);

    distance = pulseIn(Echo, HIGH) / 58.00;
    Serial.println(distance);
	return distance;
}

/*
* Function: Obstacle avoidance
* Parameter: set_dis sets the obstacle avoidance distance
*/
void avoidance(int set_dis)
{
    int angle;
    int dis[3];// distance
  
    motor(forward,MotSpeed1,MotSpeed2);

    servo(48); //Steering engine back to center
    //Obtain the distance between the cart and the obstacle and store it in dis[1]
    dis[1] = GetDistance(); 

    if(dis[1] < set_dis )
    {
        motor(stop,0,0 );   //Stop the car

        //If the left and right infrared obstacle avoidance sensors do not encounter obstacles when the steering gear rotates
        
            for (angle = 48; angle <= 100; angle++) 
            {
                servo(angle);
                delay(1);         
            }
           // Measure the distance between the left obstacle and the cart, and store the measurement data in dis[2]
            dis[2]=GetDistance(); 

            for (angle = 48; angle >= 10; angle--) 
            {
                servo(angle);  
                delay(1);             
                if(angle == 48)  
                {   // Stores measurement data in dis[1]
                    dis[1] = GetDistance(); 
                } 
            }
           // Record the range data on the right side of the trolley
            dis[0] = GetDistance(); 
            for (angle = 10; angle <= 48; angle++) 
            {
                servo(angle);
                delay(1); 
            }
        

        if(dis[0] < dis[2] ) 
        {
            motor(back,MotSpeed1,MotSpeed2); // back
            delay(300);
//            motor(left,MotSpeed1,MotSpeed2);  //left
            motor(right,MotSpeed1,MotSpeed2); 
            delay(300);
        }
        //The right is more distant from the obstacle than the left
        else if (dis[0] > dis[2] )
        {
            motor(back,MotSpeed1,MotSpeed1);
            delay(300);
            motor(right,MotSpeed1,MotSpeed2);  
            delay(300);
        } 
    }
}

/*
M3 M4 motor drive
M3 ------> speed1
M4 ------> speed2
Speed adjustment range: 0~255
*/
void motor(int dir ,int speed1, int speed2)
{  
    switch (dir)
    {
    case stop:
        motor3(0, 0);
        motor4(0, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case forward:
       
        motor3(speed1, 0);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 150, 0));
        pixels.setPixelColor(1, pixels.Color(0, 150, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case back:
        
        motor3(0, speed1);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(200, 0, 0));
        pixels.setPixelColor(1, pixels.Color(200, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case right:
        motor3(0, speed1);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0)); //LED out on the right
        pixels.setPixelColor(1, pixels.Color(255, 255, 0)); //The left LED lights up
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case left:
        
        motor3(speed1, 0);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(255, 255, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;
    
    default:
        break;
    }
}

void motor1(bool PinState1, bool PinState2)
{
   digitalWrite(12,PinState1);
   digitalWrite(4,PinState2); 
}

void motor2(bool PinState1, bool PinState2)
{
   digitalWrite(7,PinState1);
   digitalWrite(8,PinState2); 
}

void motor3(int speed1, int speed2)
{
   analogWrite(5,speed1);
   analogWrite(6,speed2); 
}

void motor4(int speed1, int speed2)
{
   analogWrite(9,speed1);
   analogWrite(10,speed2); 
}

//Customize a steering gear function
void servo(int angle) 
{ 
    //Sends 5 pulses
    for(int i=0;i<5;i++)
    {
      //Convert the Angle to a pulse width of 500-2480
        int pulsewidth = (angle * 11) + 500; 
        //Turn steering gear interface level to high
        digitalWrite(servoPin, HIGH);
        //The number of microseconds of delay pulse width value   
        delayMicroseconds(pulsewidth);  
        //Turn steering gear interface level to low
        digitalWrite(servoPin, LOW);    
        delayMicroseconds(20000 - pulsewidth);
    }
}
