#include <Adafruit_NeoPixel.h>

//Define the direction of car movement
#define  stop     0
#define  forward  1
#define  back     2
#define  left     3
#define  right    4

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN 13 // On Trinket or Gemma, suggest changing this to 1

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS 5 // Popular NeoPixel ring size

// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

static void (*reset_this_CPU)(void) = 0x0000;

#define DELAYVAL 500 // Time (in milliseconds) to pause between pixels

int MotSpeed1 = 200; 
int MotSpeed2 = 200;

void setup()
{
    Serial.begin(9600); 

   
    pinMode(5, OUTPUT);
    pinMode(6, OUTPUT);
    pinMode(9, OUTPUT);
    pinMode(10, OUTPUT);
    
    pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)

}

void loop()
{
   
    bluetooth();
     
}

void bluetooth()
{
    char serialData;   
    static char cmd;

    if(Serial.available() > 0)      
    {
        serialData = Serial.read(); 
        Serial.println(serialData); 
       
        SetSeep(serialData);

        if ('F' == serialData )  
        {
            motor(forward,MotSpeed1,MotSpeed2);
        }     
        
        else if('B' == serialData )  
        {
            motor(back,MotSpeed1,MotSpeed2);
        }
        else if ('L' == serialData )  
        {
            motor(left,MotSpeed1,MotSpeed2);
        }     
        
        else if('R' == serialData )  
        {
            motor(right,MotSpeed1,MotSpeed2);
        }
        
        else if('S' == serialData )  
        {
            motor(stop,0,0);
            cmd = 'S';
        } 
        else if('T' == serialData )  
        {
            cmd = 'T';
        }
        else if('A' == serialData )  
        {
            cmd = 'A';
        }
         
        else if('K' == serialData )  
        {
            reset_this_CPU();   //Restart the program
        }
         
    }   
}


/*
M3 M4 motor drive
M3 ------> speed1
M4 ------> speed2
Speed adjustment range: 0~255
*/
void motor(int dir ,int speed1, int speed2)
{  
    switch (dir)
    {
    case stop:
        motor3(0, 0);
        motor4(0, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case forward:
       
        motor3(speed1, 0);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 150, 0));
        pixels.setPixelColor(1, pixels.Color(0, 150, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case back:
        
        motor3(0, speed1);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(200, 0, 0));
        pixels.setPixelColor(1, pixels.Color(200, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case left:
        motor3(0, speed1);
        motor4(0, speed2);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(0, 0, 0)); //LED out on the right
        pixels.setPixelColor(1, pixels.Color(255, 255, 0)); //The left LED lights up
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;

    case right:
        
        motor3(speed1, 0);
        motor4(speed2, 0);
        pixels.clear(); // Set all pixel colors to 'off'
        pixels.setPixelColor(0, pixels.Color(255, 255, 0));
        pixels.setPixelColor(1, pixels.Color(0, 0, 0));
        pixels.show();   // Send the updated pixel colors to the hardware.
        break;
    
    default:
        break;
    }
}

void motor1(bool PinState1, bool PinState2)
{
   digitalWrite(12,PinState1);
   digitalWrite(4,PinState2); 
}

void motor2(bool PinState1, bool PinState2)
{
   digitalWrite(7,PinState1);
   digitalWrite(8,PinState2); 
}

void motor3(int speed1, int speed2)
{
   analogWrite(5,speed1);
   analogWrite(6,speed2); 
}

void motor4(int speed1, int speed2)
{
   analogWrite(9,speed1);
   analogWrite(10,speed2); 
}

void SetSeep(char val)
{
    if(val == '+')
    {
        if( MotSpeed1 < 255)
        {
            MotSpeed1 += 10;
            MotSpeed2 = MotSpeed1;

            if(MotSpeed1 > 255) //Prevent exceeding 255
            {
                MotSpeed1 = 255;
                MotSpeed2 = 255;
            }  
        }
        Serial.print("speed = ");  
        Serial.println(MotSpeed1); 
    }
    if (val == '-')
    {
        if( MotSpeed1 > 100) //Limit speed minimums
        {
            MotSpeed1 -= 10;
            MotSpeed2 = MotSpeed1;
        }

        Serial.print("speed = ");  
        Serial.println(MotSpeed1);  
    }
}
